// $Id: fsm2dot.C,v 1.4 2001/08/08 12:33:32 dvermeir Exp $
#include <fstream>

#include "fsm.h"
#include "parser.h"

int
main(int argc, char* argv[]) {
static const string USAGE("fsm2dot -- translate fsm to dot format filter");
if (argc!=1) {
  cerr << USAGE << endl;
  return 1;
  }

// Input automaton.
ref<Fsm> fsm = Parser(cin).parse();

if (!fsm) {
  cerr << "Syntax error in automaton description" << endl;
  return 2;
  }

// Generate dot file.

// Trick: use special ``0'' node with not label and no shape and an
// arrow from 0 to the initial state. This will show up as an arrow
// coming from nowhere to the initial state, the convention for the
// graphical representation of an automaton.

cout << "digraph G {\n"
     << "0 [shape=plaintext label=\"\"];\n"
     << "0 -> " << fsm->initial_state() << ";\n";

// Draw an edge for each transition.
for (Fsm::TRANSITIONS::const_iterator t=fsm->begin(); t!=fsm->end(); ++t)
  for (Fsm::STATES::const_iterator q=t->second.begin(); q!=t->second.end(); ++q)
    cout << (*t).first.first << " -> " << (*q)
         << " [label=\"" << (*t).first.second << "\"];\n";

// Use doublecircle shape for final states.
for (Fsm::STATES::const_iterator f=fsm->fbegin(); f!=fsm->fend(); ++f)
  cout << *f << " [shape=doublecircle];\n";

cout << "}\n";

return 0;
}
