#ifndef FSM_DFA_H
#define FSM_DFA_H
// $Id: mkdfa.h,v 1.6 2001/08/18 14:22:50 dvermeir Exp $

#include <vector>
#include <iostream>

#include "fsm.h"

//! Class that can make a dfa out of a fsm.
/*! The states of the new machine are all of the form ``q123''
    where 123 is a positive integer.

    Normal usage:
    \code
    Fsm*	fsm;
    ref<Fsm>	dfsm = DfaMaker(fsm).mkdfa();

    if (!dfsm) 
      cerr << "Error making nfa deterministic.";
    \endcode
*/
class DfaMaker {
public:
  //! Constructor, first argument is machine to make deterministic.
  /*! If debugstream is not 0, some debugging output will be
      written to it.
  */
  DfaMaker(const Fsm& fsm, ostream *debug=0);

  //! Return pointer to newly allocated dfa equivalent with fsm_.
  ref<Fsm> mkdfa();

  //! Generate name of the form ``q123'' where ``123'' is string rep. of i.
  static Fsm::State mkstate(int i);

private:
  //! Compute the delta(Q,a) where Q is a set of ``old'' states.
  /*!  Fill up R to contain the union of all 
       <code>fsm_.delta(q,a)</code> 
       for q in Q.
  */
  Fsm::STATES& delta(const Fsm::STATES& Q, 
  		     const Fsm::Symbol& a, Fsm::STATES& R);

  //! Expand Q so that it is closed under epsilon-transitions.
  Fsm::STATES& closure(Fsm::STATES& Q) const;

  //! Does Q contain a final state from fsm_?
  bool is_final(const Fsm::STATES& Q) const;

  //! Machine to make deterministic.
  const Fsm&	fsm_;
  //! If true, some debug output may be generated.
  ostream* 	debug_;
};

#endif
