#ifndef NOCOMMENT_STREAM_H
#define NOCOMMENT_STREAM_H
// $Id: nocomment.h,v 1.1 2001/08/07 09:16:00 dvermeir Exp $

#include <string>
#include <iostream>
#include "filterstreambuf.h"

/*! \file
    This files defines a class nocommentstream, which uses
    a filterstreambuf to remove shell style comments.
*/

//! A nocommentstream is an istream that strips '#' comments.
/*! All characters following (and including) a '#' are ignored
    up to (but not including) the next newline character.
*/
class nocommentstream: public istream {
private:
  //! Implements Filter interface for FilterStreambuf
  class NoCommentFilter {
  public:
    NoCommentFilter(istream& is): is_(is) {}
    ~NoCommentFilter() {}
    int put(int c) { return EOF; }
    int get();
    int sync() { return -1; }
    void close() {}
  private:
    //! Underlying istream from where the filter takes its input.
    istream&	is_;
  };
public:
  //! Uses open stream.
  nocommentstream(istream& stream);
  //! Destructor.
  ~nocommentstream() { delete rdbuf(); }
private:
  NoCommentFilter	filter_;
};

#endif
