#ifndef FSM_OPTIMIZER_H
#define FSM_OPTIMIZER_H
// $Id: optimizer.h,v 1.4 2001/08/08 12:33:33 dvermeir Exp $

#include <iostream>

#include "fsm.h"

//! Class that encapsulates the algorithm to generate an optimal dfa.
/*! It relies on the Fsm interface to query the input dfa and
    to build the new optimal dfa.
*/
class Optimizer {
public:
  //! Constructor, if trace is not 0, debug info will be written on it,
  Optimizer(const Fsm& fsm, ostream* trace = 0);

  ref<Fsm> optimize();
private:
  //! Type of distinct_ data member.
  /*! A ``triangular'' boolean matrix  representing a symmetric
      relation between states.
  */
  typedef map<pair<Fsm::State,Fsm::State>,bool> DISTINCT_T;

  //! Matrix representing equivalence of states.
  /*! If q1 and q2 are not equivalent, then distinct_[q1,q2] = true.

      Do not access directly, use the Optimizer::distinct function
      instead.
  */
  DISTINCT_T distinct_; 
  
  //! Convenient access to Optimizer::dictinct_ data member.
  /*! The arguments will be switched to match the implementation
      of the ``triangular'' matrix.
  */
  bool& distinct(const Fsm::State&, const Fsm::State&);

  //! Return representative of set of states that are equivalent to q.
  Fsm::State eq_class(const Fsm::State& q);

  //! Automaton that will be optimized.
  const Fsm&	fsm_;
  //! Stream for debug output.
  ostream*	debug_;
};

#endif
