#ifndef FSM_PARSER_H
#define FSM_PARSER_H
// $Id: parser.h,v 1.5 2001/08/08 12:33:33 dvermeir Exp $
#include <iostream>

#include "fsm.h"
#include "ref.h"
#include "nocomment.h"

//! Class that is able to parse a textual description of a FSM.
class Parser {
public:
  //! Constructor, parameter is input stream containing description.
  Parser(istream& is);
 
  //! Parse FSM description on <code>is_</code>, return 0 if syntax error.
  ref<Fsm> parse(); 
private:
  //! Parse a set of states.
  /*! Syntax is
      \code
      { state state .. }
      \endcode
  */
  bool parse_states(Fsm::STATES&);
  //! Parse a set of symbols.
  /*! Syntax is
      \code
      { symbol symbol .. }
      \endcode
  */
  bool parse_syms(Fsm::SYMBOLS&);
  //! Parse a transition.
  /*! Syntax is
      \code
      states symbols states
      \endcode
  */
  bool parse_transition(Fsm::STATES&, Fsm::SYMBOLS&, Fsm::STATES&);

  //! Input stream of parser.
  nocommentstream is_;
};
#endif
